#include <stdio.h>
#include <math.h>
#include <GL/glut.h>

int n=25, m=8;

int s=4, t=4;

int lines=1, polys=1;

int verbose=0;

#define R(i,j) M_PI*(2*(i)+((j)%2))/n
#define THETA(i,j) (double)(j)/(m-1)

#define STEP(x1,x2,count,max) (x1+(x2-x1)*((double)(count)/max))
#define XY(r,theta) (sin(r)*(theta)),(cos(r)*(theta))

void init(void)
{
#if 0
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA); /* from manual... */
	glEnable(GL_BLEND);
	glEnable(GL_LINE_SMOOTH);
#endif
}

void point(int i, int j, int poly)
{
	int z;
	double r1, th1;
	double r2, th2;
	double r3, th3;
	double r, th;
	r1=R(i,j+1);
	th1=THETA(i,j+1);
	r2=R(i+1,j+1);
	th2=THETA(i+1,j+1);
	r3=R((j%2)?i:i+1,j);
	th3=THETA((j%2)?i:i+1,j);
	
	r=(r1+r2+r3)/3;
	th=(th1+th2+th3)/3;

	if (poly)
		glVertex2f(XY(r,th));

	for(z=0;z<s;z++)
		glVertex2f(XY(STEP(r1,r2,z,s),STEP(th1,th2,z,s)));
	for(z=0;z<t;z++)
		glVertex2f(XY(STEP(r2,r3,z,t),STEP(th2,th3,z,t)));
	for(z=0;z<t;z++)
		glVertex2f(XY(STEP(r3,r1,z,t),STEP(th3,th1,z,t)));
	
	glVertex2f(XY(r1,th1));
}

void display()
{
	int i,j;
	glClear(GL_COLOR_BUFFER_BIT);

	for(i=0;i<n;i++) {
		for(j=0;j<m-1;j++) {
			if (polys) {
				glColor3f(.5,.5,.5);
				glBegin(GL_TRIANGLE_FAN);
				point(i,j,1);
				glEnd();
			}
			if (lines) {
				glColor3f(1,1,1);
				glBegin(GL_LINE_STRIP);
				point(i,j,0);
				glEnd();
			}
		}
	}
	
	glFlush();
}

void reshape(int width, int height)
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void key(unsigned char key, int x, int y)
{
	switch (key) {
	case 27:
		exit(0);
		break;
	case '-':
		n--;
		glutPostRedisplay();
		break;
	case '=':
	case '+':
		n++;
		glutPostRedisplay();
		break;
	case 'a':
		m--;
		glutPostRedisplay();
		break;
	case 's':
		m++;
		glutPostRedisplay();
		break;
	case 'q':
		s--;
		glutPostRedisplay();
		break;
	case 'w':
		s++;
		glutPostRedisplay();
		break;
	case 'e':
		t--;
		glutPostRedisplay();
		break;
	case 'r':
		t++;
		glutPostRedisplay();
		break;
	case 'l':
		lines^=1;
		glutPostRedisplay();
		break;
	case 'p':
		polys^=1;
		glutPostRedisplay();
		break;
	case 'v':
		verbose^=1;
		break;
	}
	if (verbose)
		printf("%d %d %d %d %d\n", n, m, s, t, n*m*(s+2*t));
}

int main( int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitWindowSize(600, 600);

	glutInitDisplayMode(GLUT_RGB);
	
	glutCreateWindow(*argv);
	
	glutKeyboardFunc(key);
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	
	init();

	glutMainLoop();
}
